package com.evermind.ejb.taglib;

import javax.ejb.*;
import javax.servlet.http.*;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.*;

/**
 * UseBeanTag
 * 
 * @author Magnus Stenman
 */
public class UseBeanTag extends TagSupport
{
	protected Object value;
	protected String type;
	protected String scope;

	public void setScope(String scope)
	{
		this.scope = scope;
	}

	public void setValue(Object value)
	{
		this.value = value;
	}

	public void setType(String type)
	{
		this.type = type;
	}

	public int doStartTag() throws JspException
	{
		if(value != null)
		{
			setValue();

			if(scope != null)
			{
				if(scope.equals("session"))
					pageContext.setAttribute(this.getId(), value, PageContext.SESSION_SCOPE);
				else if(scope.equals("request"))
					pageContext.setAttribute(this.getId(), value, PageContext.REQUEST_SCOPE);
				else if(scope.equals("application"))
					pageContext.setAttribute(this.getId(), value, PageContext.APPLICATION_SCOPE);
			}
				
			return SKIP_BODY;
		}
		else if(scope != null)
		{
			if(scope.equals("session"))
			{
				HttpSession session = pageContext.getSession();
				
				if(session != null)
				{
					Object object = session.getAttribute(this.getId());
				
					if(object instanceof EJBObject)
					{
						pageContext.setAttribute(this.getId(), object);
						return SKIP_BODY;
					}
				}
			}
			else if(scope.equals("request"))
			{
				Object object = pageContext.getRequest().getAttribute(this.getId());
				
				if(object instanceof EJBObject)
				{
					pageContext.setAttribute(this.getId(), object);
					return SKIP_BODY;
				}
			}
			else if(scope.equals("application"))
			{
				Object object = pageContext.getServletContext().getAttribute(this.getId());
				
				if(object instanceof EJBObject)
				{
					pageContext.setAttribute(this.getId(), object);
					return SKIP_BODY;
				}
			}
		}

		return EVAL_BODY_INCLUDE;
	}
	
	private void setValue() throws JspException
	{
		try
		{	
			if(!(value instanceof EJBLocalObject))
				value = (EJBObject)javax.rmi.PortableRemoteObject.narrow(value, Class.forName(type, true, pageContext.getPage().getClass().getClassLoader()));

			pageContext.setAttribute(this.getId(), value);
		}
		catch(ClassNotFoundException e)
		{
			throw new JspException("ClassNotFoundException: " + e.getMessage());
		}
	}
	
	public int doEndTag() throws JspException
	{
		return EVAL_PAGE;
	}
}
